/* 
        MmsSvgaWindow.cc

        MASL/M-MASL Copyright (c) KUNI/TDU
        Windows version Copyright (c) 1995, 1996 Tomoatsu Shimada  (shimarin)
        X Window System version Copyright (c)1997 Yamahata Isaku (ISA-9)
*/
#include <assert.h>
#include <vgakeyboard.h>

#include "MmsConst.h"
#include "MmsSvgaWindow.h"
#include "getScanCode.h"

MmsSvgaWindow::MmsSvgaWindow (XMMS_OPTION* mmsXmmsOption):
  gSvgaMode (G640x480x256),
  imageDataOf2x0LineMode (NULL),
  xmmsOption (mmsXmmsOption)
{
  vga_init ();
  vga_runinbackground (0);//1=enable
  setSvgaMode (gSvgaMode);
  physicalScreen=gl_allocatecontext ();
  gl_getcontext (physicalScreen);
  if (keyboard_init ()==-1)
    {
      cerr << "Could not initialize keyboard.\n";
      vga_setmode (TEXT);
      exit(-1);
    }
  keyboard_translatekeys(TRANSLATE_CURSORKEYS|
			 TRANSLATE_DIAGONAL|
			 TRANSLATE_KEYPADENTER);
  getScanCode_init ();
  
  graphicMode=GM640x480;
  for (int i=0;i<NUM_OF_WINDOW;i++)
    virtualWindow[i].resetVariable ();

  windowLeft=0;
  windowTop=0;
  windowWidth=WINDOW_SIZE_X;
  windowHeight=WINDOW_SIZE_Y;
  
  //̾ν
  int imageSizeX=WINDOW_SIZE_X;
  int imageSizeY=WINDOW_SIZE_Y;
  if (xmmsOption->isShowVirtualVram)
    {//ɽwindowvirtual vramƤɽ롣
      //Τν
      imageSizeX=VIRTUAL_VRAM_SIZE_X;
      imageSizeY=VIRTUAL_VRAM_SIZE_Y;
    }
  imageData=new char[imageSizeX*imageSizeY];
}

MmsSvgaWindow::~MmsSvgaWindow ()
{
  keyboard_close ();
  vga_setmode (TEXT);
  gl_freecontext (physicalScreen);

  delete[] imageData;
  delete[] imageDataOf2x0LineMode;
}

void MmsSvgaWindow::setGraphicMode (GraphicMode mmsGraphicMode)
{
  graphicMode=mmsGraphicMode;
  //ximagewindowΤɤΰ֤ˤɤ礭ɽ뤫ꤹ롣
  //window˲ɽͤ˰֤ꡣ
  switch (graphicMode)
    {//礭
    case GM320x200:
      windowWidth=320;
      windowHeight=200;
      if (xmmsOption->isReduction)
	{
	  windowLeft=(640-320)/2;
	  windowTop=(480-200)/2;
	  setSvgaMode (G640x480x256);
	}
	else
	{
	  windowLeft=0;
	  windowTop=0;
	  setSvgaMode (G320x200x256);
	}
      break;
    case GM320x240:
      windowWidth=320;
      windowHeight=240;
      if (xmmsOption->isReduction)
	{
	  windowLeft=(640-320)/2;
	  windowTop=(480-320)/2;
	  setSvgaMode (G640x480x256);
	}
      else
	{
	  windowLeft=0;
	  windowTop=0;
	  setSvgaMode (G320x240x256);
	}
      break;
    case GM640x400: 
      windowWidth=640;
      windowHeight=400;
      windowLeft=0;
      windowTop=(480-400)/2;
      setSvgaMode (G640x480x256);
      break;
    case GM640x480: 
      windowWidth=640;
      windowHeight=480;
      windowLeft=0;
      windowTop=0;
      setSvgaMode (G640x480x256);
      break;
    default:
      break;//do nothing;
    }
  memset (imageData,0,WINDOW_SIZE_X*WINDOW_SIZE_Y*sizeof (char));
}

void MmsSvgaWindow::setWindowSize (int x1,int y1,int x2,int y2,int n=0)
{
  assert (x1>=0);
  assert (x1<WINDOW_SIZE_X);
  assert (y1>=0);
  assert (y1<WINDOW_SIZE_Y);
  assert ((x1+x2)+(y1+y2)*WINDOW_SIZE_X<WINDOW_SIZE_X*WINDOW_SIZE_Y);
  if (y1+y2>=WINDOW_SIZE_Y)
    y2=WINDOW_SIZE_Y-y1-1;

  virtualWindow[n].vramX=x1;
  virtualWindow[n].vramY=y1;
  virtualWindow[n].width=x2;
  virtualWindow[n].height=y2;
}

void MmsSvgaWindow::moveWindow (int x,int y,int n=0)
{
  //////////////////////////////////virtual vramϤߤʤå
  assert (x>=0);
  assert (x<VIRTUAL_VRAM_SIZE_X);
  assert (y>=0);
  assert (y<VIRTUAL_VRAM_SIZE_Y);
  assert ((x+virtualWindow[n].width)+
	  (y+virtualWindow[n].height)*VIRTUAL_VRAM_SIZE_X
	  <VIRTUAL_VRAM_LENGTH);
  if (x<0)
    x=0;
  if (x>=VIRTUAL_VRAM_SIZE_X)
    x=VIRTUAL_VRAM_SIZE_X-1;
  if (y<0)
    y=0;
  if (y>=VIRTUAL_VRAM_SIZE_Y)
    y=VIRTUAL_VRAM_SIZE_Y-1;
  if (y+virtualWindow[n].height>=VIRTUAL_VRAM_LENGTH)
    virtualWindow[n].height=VIRTUAL_VRAM_SIZE_Y-y-1;

  virtualWindow[n].virtualVramX=x;
  virtualWindow[n].virtualVramY=y;
}

void MmsSvgaWindow::updateWindow (const unsigned char* const mmsImageData)
  //̤κmethod
{
  if (xmmsOption->isTraceMode)
    {//////trace modeäΤǤν
      //keyboard_waitforupdate ();//ȲΤconsoleȤ˴ʤˤʤ롣
#if 0
      keyboard_update ();
      char* tmp=keyboard_getstate ();
      for (int i=0;i<106;i++)
	cout << i << "=" << (int) tmp[i];
      cout << endl;
#endif
    }

  putImage (mmsImageData);//ɽ
}

void MmsSvgaWindow::putImage ()
{//////////////windowɽ
  switch (graphicMode)
    {
    case GM320x200:
      putImage320x200 ();
      break;
    case GM320x240:
      putImage320x240 ();
      break;
    case GM640x400: 
      putImage640x400 ();
      break;
    case GM640x480: 
      putImage640x480 ();
      break;
    default:
      break;//do nothing;
    }

}

void MmsSvgaWindow::putImage (const unsigned char* const mmsImageData)
{
  //virtual vramɽѤwindowvirtual vramƤɽ롣
  vga_lockvc ();
  if (!vga_oktowrite ())
    {
      vga_unlockvc ();
      return;
    }
  //vga_waitretrace ();  
  /////////////dataž
  for (int i=0;i<NUM_OF_WINDOW;i++)
    {
      if (virtualWindow[i].width==0 || virtualWindow[i].height==0)
	continue;//windowĤƤ褷ʤ
      
      gl_putboxpart (windowLeft+virtualWindow[i].vramX,
		     windowTop+virtualWindow[i].vramY,
		     virtualWindow[i].width,virtualWindow[i].height,
		     VIRTUAL_VRAM_SIZE_X,VIRTUAL_VRAM_SIZE_Y,
		     mmsImageData,
		     virtualWindow[i].virtualVramX,
		     virtualWindow[i].virtualVramY);
#if 0      
      //ޤdataΤɤˤʤ뤫offset׻Ƥ
      int vramOffset=virtualWindow[i].vramX+
	virtualWindow[i].vramY*WINDOW_SIZE_X;
      int virtualVramOffset=virtualWindow[i].virtualVramX+
	virtualWindow[i].virtualVramY*VIRTUAL_VRAM_SIZE_X;
      //private colormapѤƤΤǡpalette==pixel value
      //ʤΤǤΤޤͤcopyɤ
      for (int y=0;y<virtualWindow[i].height;y++)
	{
	  memcpy (imageData+vramOffset+y*WINDOW_SIZE_X,
		  mmsImageData+virtualVramOffset+y*VIRTUAL_VRAM_SIZE_X,
		  virtualWindow[i].width);
	}
#endif
    }
  vga_unlockvc ();
  //putImage ();
  //mmsXPutImage (imageData);
}

void MmsSvgaWindow::putImage320x200 ()
{
  if (xmmsOption->isReduction)
    {//򤻤window
      mmsXPutImage (imageData);
      return;
    }


  if (imageDataOf2x0LineMode==NULL)
    allocXimageOf2x0LineMode ();//ƸƤФ줿ΤǡѴѤXImageݤ
  
  ///G320x200x256modeѤѴ롣
  for (int y=0;y<200;y++)
    {
      memcpy (imageDataOf2x0LineMode+y*WINDOW_SIZE_X/2,
	      imageData+y*WINDOW_SIZE_X,
	      WINDOW_SIZE_X/2);
    }

#if 0
  //320x200line modeʤΤǺ320x200640x400˳
  for (int y=0;y<200;y++)
    {
      for (int x=0;x<320;x++)
	{
	  imageDataOf2x0LineMode[x*2+y*2*WINDOW_SIZE_X]=
	    imageData[x+y*WINDOW_SIZE_X];
	  imageDataOf2x0LineMode[x*2+1+y*2*WINDOW_SIZE_X]=
	    imageData[x+y*WINDOW_SIZE_X];
	}
      memcpy (imageDataOf2x0LineMode+(y*2+1)*WINDOW_SIZE_X,
	      imageDataOf2x0LineMode+(y*2)*WINDOW_SIZE_X,
	      WINDOW_SIZE_X);
    }
#endif
  ///window
  mmsXPutImage (imageDataOf2x0LineMode);
}

void MmsSvgaWindow::putImage640x400 ()
{
  ////Ѵɬ̵

  ///window
  mmsXPutImage (imageData);
}

void MmsSvgaWindow::putImage320x240 ()
{
  if (xmmsOption->isReduction)
    {//򤻤window
      mmsXPutImage (imageData);
      return;
    }

  if (imageDataOf2x0LineMode==NULL)
    allocXimageOf2x0LineMode ();//ƸƤФ줿ΤǡѴѤXImageݤ

  ///G320x240x256modeѤѴ롣
  for (int y=0;y<240;y++)
    {
      memcpy (imageDataOf2x0LineMode+y*WINDOW_SIZE_X/2,
	      imageData+y*WINDOW_SIZE_X,
	      WINDOW_SIZE_X/2);
    }
#if 0
  ////320x240modeʤΤǺ320x240640x480˳
  for (int y=0;y<240;y++)
    {
      for (int x=0;x<320;x++)
	{
	  imageDataOf2x0LineMode[x*2+y*2*WINDOW_SIZE_X]=
	    imageData[x+y*WINDOW_SIZE_X];
	  imageDataOf2x0LineMode[x*2+1+y*2*WINDOW_SIZE_X]=
	    imageData[x+y*WINDOW_SIZE_X];
	}
      memcpy (imageDataOf2x0LineMode+(y*2+1)*WINDOW_SIZE_X,
	      imageDataOf2x0LineMode+(y*2)*WINDOW_SIZE_X,
	      WINDOW_SIZE_X);
    }
#endif
  ///window
  mmsXPutImage (imageDataOf2x0LineMode);
}

void MmsSvgaWindow::putImage640x480 ()
{
  ////Ѵɬפ̵

  ///window
  mmsXPutImage (imageData);
}

void MmsSvgaWindow::allocXimageOf2x0LineMode ()
{
  imageDataOf2x0LineMode=new char[WINDOW_SIZE_X*WINDOW_SIZE_Y];
}
