/* 
        Scenario.h

        MASL/M-MASL Copyright (c) KUNI/TDU$B1~>p8&(B
        Windows version Copyright (c) 1995, 1996 Tomoatsu Shimada  (shimarin)
        X Window System version Copyright (c)1997 Yamahata Isaku (ISA-9)
*/
#include <string.h>
//#include <cstring>
#include <ctype.h>
//#include <cctype>
#include <assert.h>
//#include <cassert>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "MaslInputStream.h"

#ifndef scenario_h
#define scenario_h
class Scenario
{
protected:
  int scenarioSize;
  char* scenarioData;//$B%7%J%j%*$N@hF,$r;X$9(Bpointer(header$B4^$`(B)
  int ver1;//major version
  int ver2;//minor version
  const char* comment;//comment$B$N@hF,$r;X$9(Bpointer
  int commentSize;//commentSize$B$O%3%a%s%HNN0h$N$=$NBg$-$5(B
  int docwait;
  //$B%3%a%s%H$rI=<($7$F$+$i%"%K%a!<%7%g%s:F@8$^$G$NBT$A;~4V(B($BC10L$OIC(B)
  char* dataFilename;//scenario$B$N%U%!%$%kNN0h$r;X$9(Bpointer
  char* reserved;
  char* beginPoint;//scenario$B$NL?Na$,;O$^$k$H$3$m$N@hF,$r;X$9!#(B
  char* pt;//$B8=:_;X$7$F$$$k>l=j(B
  int searchnum (const char*& tmp);
  inline int searchnum (){return searchnum (pt);};
  virtual void init ()=0;

  Scenario (char* filename);
  Scenario (MaslInputStream& mmsIs,int mmsScenarioSize);
  Scenario (char* data,int mmsScenarioSize);
public:
  virtual ~Scenario ();
  virtual void showInfo ()=0;//$B>pJs$rI8=`=PNO$KI=<($9$k!#(B
  virtual Bool checkMagic ()=0;//magic number$B$,@5$7$$$+%A%'%C%/(B
  void printComment (Bool isSjis);//(M-)MASL$B$N%3%a%s%HItJ,$rI8=`=PNO$KI=<((B
  //isSjis=True$B$J$i2?$bJQ49$;$:$KI=<(!#(B
  //isSjis==False$BF`NI(Bsjis->euc$B$NJQ49$r9T$C$F$+$iI=<((B
  const char* const getDataFilename ();
  //.msl,.mms$B$,4^$`(B*.ak5,*.nak,*.pkn$B$r$N%U%!%$%k%M!<%`$r=g$KF@$k!#(B
  //$BA4$F=P$7$?8e$O(BNULL$B$rJV$9!#(B
  const char* const getReservedName ();
  //$BM=LsNN0h$K=q$+$l$F$$$k%U%!%$%k%M!<%`$r=g$KF@$k!#(B
  //$BA4$F=P$7$?8e$O(BNULL$B$rJV$9!#(B
  Bool searchNextCommand ();
  static int convertSjis2Euc (const unsigned char* const comment,
			      const int commentSize,
			      unsigned char* const eucComment);
  //comment$B$KF~$C$F$$$kD9$5(BcommentSize byte$B$N(Bsjis$B$r$K(Beuc$B$KJQ49$7$F(B
  //ecuComment$B$K$$$l$FJV$9!#(B  eucComment$B$O==J,$JD9$5$r3NJ]$7$F$*$/$3$H!#(B
  //(commentSize*2)byte
  static ostream& writeSjis (ostream& outputStream,
			     const char* const sjisData,const int size,
			     const Bool isOutputSjis=False);
  //out$B$K(Bwrite$B$9$k!#(BisOutputSjis=False$B$N;~$O(Bsjis$B$r(Beuc$B$KJQ49$9$k!#(B
  //                isOutputSjis=True$B$N;~$O$=$N$^$^(B

  ////////////accsess method
  inline int getScenarioSize () const {return scenarioSize;};
  //$B%7%J%j%*$N%5%$%:$r%P%$%H$GJV$9(B
  inline int getVersion () const {return ver1*10000+ver2;};
  //masl,m-masl$B$N(Bversion$B;XDj$rJV$9!#(B
  inline int getDocWaitValue () const {return docwait;};
  //$B:F@8A0$K2?ICBT$D$+;XDj$5$l$F$$$kIC?t$rJV$9!#(B
  //$B;XDj$5$l$F$$$J$1$l$PL58B$@$,<B:]$O(B32767(4byte int$B$N:GBg(B)$B$rJV$9!#(B
  inline const char* getScenarioTop () const {return scenarioData;};
  //$B%7%J%j%*$N:G=i$rJV$9!#$D$^$j(Bversion$B;XDj$N$"$k9T$N<!$N9T$N@hF,(B
  inline char* getBeginPoint () const{return beginPoint;};
  //scenario$B$N$&$AL?Na$,=q$+$l$F$$$kNN0h$N:G=i$rJV$9!#(B
  inline int getCommandLength () const
    {return scenarioSize-(scenarioData-beginPoint);};
  //$BL?Na$,=q$+$l$F$$$kNN0h$NBg$-$5$r%P%$%H$GJV$9!#(B
  inline void skip (int c=1)
    {
      assert (pt+c<=scenarioData+scenarioSize);
      pt+=c;
    };
  inline char getChar ()
    {
      return *pt;
    };
  inline char getChar (int offset)
    {
      assert (pt+offset<=scenarioData+scenarioSize);
      return *(pt+offset);
    };
  inline const char* getCurrentPointer () const {return pt;};
  inline int getDigitValue (){return searchnum ();};
  inline Bool isChar (char ch) const {return (ch==*pt);};
  inline Bool isString (const char* const string) const
    {
      assert (pt+strlen (string)<=scenarioData+scenarioSize);
      return (strncmp (pt,string,strlen (string))==0);
    };
  inline Bool isDigit () const { return isdigit (*pt);};
  inline Bool isValue () const
    {return isdigit (*pt)||*pt=='R'||*pt=='@'||*pt=='K'||*pt=='O'||*pt=='P'||
       *pt=='F';};
  inline void setBeginPoint () {pt=beginPoint;};
  inline Bool isEnd (const char* now) const
    {return now>=scenarioData+scenarioSize;};
  inline Bool isNotEnd (const char* now) const
    {return now<scenarioData+scenarioSize;};
};
#endif scneario_h
